unit CaptionButton;

{-------------------------------------------------------------------------------
  Copyright 1999, 2000 Jason Plackey
  jplackey@integrationware.com
  - Freeware -

  TCaptionButton is a small control intended to provide an easy way to achieve
  the behavior of Windows' standard caption buttons anywhere on your form.  You
  can select any of the available button types (close, help, maximize, minimize,
  or restore).

  System metrics are used to set the original control size, but the buttons may
  be resized as desired; DrawFrameControl will scale the glyph accordingly.  The
  ForceSystemMetrics property can be used, however, to prevent sizing of the
  control.  The control responds to CM_WININICHANGE (essentially
  WM_SETTINGCHANGED), and when ForceSystemMetrics is True, the control will
  adjust its size to accomodate the new system metrics.
-------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TCaptionButtonType = ( cbtClose, cbtHelp, cbtMax, cbtMin, cbtRestore );

type
  TCaptionButton = class(TCustomControl)
  private
    FButtonType: TCaptionButtonType;
    FInClient : Boolean;
    FEnabled : Boolean;
    FForceMetrics: Boolean;
    procedure SetButtonType(const Value: TCaptionButtonType);
    procedure _SetEnabled(const Value: Boolean);
    procedure SetForceMetrics(const Value: Boolean);
  protected
    procedure SetControlSize;
    procedure Paint; override;
    procedure CMMouseEnter( var Message : TMessage ); message CM_MOUSEENTER;
    procedure CMMouseLeave( var Message : TMessage ); message CM_MOUSELEAVE;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure WMSize( var Message : TWMSize ); message WM_SIZE;
    procedure CMWinIniChange( var Message : TMessage ); message CM_WININICHANGE;
  public
    constructor Create( AOwner : TComponent ); override;
  published
    property ButtonType : TCaptionButtonType read FButtonType write SetButtonType;
    property ForceSystemMetrics : Boolean read FForceMetrics write SetForceMetrics;
    property Action;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled : Boolean read FEnabled write _SetEnabled;
    property ParentBiDiMode;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnClick;
    property OnContextPopup;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Additional', [TCaptionButton]);
end;

{ TCaptionButton }

procedure TCaptionButton.CMMouseEnter(var Message: TMessage);
begin
  if not FInClient then
  begin
    FInClient := True;
    Repaint;
  end;
end;

procedure TCaptionButton.CMMouseLeave(var Message: TMessage);
begin
  if FInClient then
  begin
    FInClient := False;
    Repaint;
  end;
end;

procedure TCaptionButton.CMWinIniChange(var Message: TMessage);
begin
  if ForceSystemMetrics then SetControlSize;
end;

constructor TCaptionButton.Create(AOwner: TComponent);
begin
  inherited;
  FInClient := False;
  FEnabled := True;
  FForceMetrics := True;
end;

procedure TCaptionButton.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  inherited;
  Repaint;
end;

procedure TCaptionButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;
  Repaint;
end;

procedure TCaptionButton.Paint;
var
  iStyle : Integer;
  iPushedFlag : Integer;
  iEnabled : Integer;
begin
  inherited;
  iStyle := 0;
  case ButtonType of
    cbtClose   : iStyle := DFCS_CAPTIONCLOSE;
    cbtHelp    : iStyle := DFCS_CAPTIONHELP;
    cbtMax     : iStyle := DFCS_CAPTIONMAX;
    cbtMin     : iStyle := DFCS_CAPTIONMIN;
    cbtRestore : iStyle := DFCS_CAPTIONRESTORE;
  end;

  if Enabled then
  begin
    if (MouseCapture) and (FInClient) then iPushedFlag := DFCS_PUSHED else iPushedFlag := 0;
    iEnabled := 0;
  end
  else
  begin
    iPushedFlag := 0;
    iEnabled := DFCS_INACTIVE;
  end;

  DrawFrameControl( Canvas.Handle, ClientRect, DFC_CAPTION, iStyle or iPushedFlag or iEnabled );
end;

procedure TCaptionButton.SetButtonType(const Value: TCaptionButtonType);
begin
  FButtonType := Value;
  Repaint;
end;

procedure TCaptionButton.SetControlSize;
begin
  Width := GetSystemMetrics( SM_CXSIZE ) - ( GetSystemMetrics( SM_CXFRAME ) div 2 );
  Height := GetSystemMetrics( SM_CYSIZE ) - GetSystemMetrics( SM_CYFRAME );
end;

procedure TCaptionButton.SetForceMetrics(const Value: Boolean);
begin
  FForceMetrics := Value;
  if FForceMetrics then SetControlSize;
end;

procedure TCaptionButton.WMSize(var Message: TWMSize);
begin
  inherited;
  if ForceSystemMetrics then SetControlSize;
end;

procedure TCaptionButton._SetEnabled(const Value: Boolean);
begin
  FEnabled := Value;
  Repaint;
end;

end.
